@@ -1,7 +1,7 @@
 package App::LintPrereqs;
 
-our $DATE = '2016-02-27'; # DATE
-our $VERSION = '0.36'; # VERSION
+our $DATE = '2016-03-01'; # DATE
+our $VERSION = '0.37'; # VERSION
 
 use 5.010001;
 use strict;
@@ -22,6 +22,23 @@ require Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(lint_prereqs);
 
+# create a merged list of prereqs from any phase
+sub _create_prereqs_for_Any_phase {
+    my $prereqs = shift;
+    $prereqs->{Any}   = {};
+    for my $phase (grep {$_ ne 'Any'} keys %$prereqs) {
+        for my $mod (keys %{ $prereqs->{$phase} }) {
+            my $v = $prereqs->{$phase}{$mod};
+            if (exists $prereqs->{Any}{$mod}) {
+                $prereqs->{Any}{$mod} = $v
+                    if version_gt($v, $prereqs->{Any}{$mod});
+            } else {
+                $prereqs->{Any}{$mod} = $v;
+            }
+        }
+    }
+}
+
 sub _scan_prereqs {
     my %args = @_;
 
@@ -38,9 +55,10 @@ sub _scan_prereqs {
         }
     };
     require File::Find;
-    my @files;
+    my %files; # key=phase, val=[file, ...]
 
     {
+        $files{Runtime} = [];
         my @dirs = (grep {-d} (
             "lib", "bin", "script", "scripts",
             #"sample", "samples", "example", "examples" # decidedly not included
@@ -51,7 +69,7 @@ sub _scan_prereqs {
             sub {
                 return unless -f;
                 return if check_backup_filename(filename=>$_);
-                push @files, "$File::Find::dir/$_";
+                push @{$files{Runtime}}, "$File::Find::dir/$_";
             },
             @dirs
         );
@@ -65,71 +83,78 @@ sub _scan_prereqs {
                 return unless -f;
                 return if check_backup_filename(filename=>$_);
                 return unless /\.(t|pl|pm)$/;
-                push @files, "$File::Find::dir/$_";
+                push @{$files{Test}}, "$File::Find::dir/$_";
             },
             @dirs
         );
     }
 
-    my %res;
-    for my $file (@files) {
-        my $scanres = $scanner->scan_file($file);
-        unless ($scanres) {
-            $log->tracef("Scanned %s, got nothing", $file);
-        }
-        my $reqs = $scanres->{requirements};
-        $log->tracef("Scanned %s, got: %s", $file, [keys %$reqs]);
-        #$log->tracef("TMP:reqs=%s", $reqs);
-        for my $req (keys %$reqs) {
-            my $v = $reqs->{$req}{minimum}{original};;
-            if (exists $res{$req}) {
-                $res{$req} = $v if version_gt($v, $res{$req});
-            } else {
-                $res{$req} = $v;
+    my %res; # key=phase, value=hash of {mod=>version , ...}
+    for my $phase (keys %files) {
+        $res{$phase} = {};
+        for my $file (@{$files{$phase}}) {
+            my $scanres = $scanner->scan_file($file);
+            unless ($scanres) {
+                $log->tracef("Scanned %s, got nothing", $file);
             }
-        }
-    }
+            my $reqs = $scanres->{requirements};
+            $log->tracef("Scanned %s, got: %s", $file, [keys %$reqs]);
+            #$log->tracef("TMP:reqs=%s", $reqs);
+            for my $req (keys %$reqs) {
+                my $v = $reqs->{$req}{minimum}{original};;
+                if (exists $res{$phase}{$req}) {
+                    $res{$phase}{$req} = $v
+                        if version_gt($v, $res{$phase}{$req});
+                } else {
+                    $res{$phase}{$req} = $v;
+                }
+            }
+        } # for file
+    } # for phase
+
+    _create_prereqs_for_Any_phase(\%res);
+
     %res;
 }
 
 $SPEC{lint_prereqs} = {
     v => 1.1,
-    summary => 'Check extraneous/missing prerequisites in dist.ini',
+    summary => 'Check extraneous/missing/incorrect prerequisites in dist.ini',
     description => <<'_',
 
-Check `[Prereqs / *]` (as well as `OSPrereqs`, `Extras/lint-prereqs/Assume-*`)
-sections in your `dist.ini` against what's actually being used in your Perl code
-(using `Perl::PrereqScanner::Lite`) and what's in Perl core list of modules.
-Will complain if your prerequisites are not actually used (extraneous
-dependencies), or if there are missing prerequisites. Will also complain if a
-prerequisite is already in Perl core (but this can be configured).
+lint-prereqs can improve your prereqs specification in `dist.ini` by reporting
+prereqs that are extraneous (specified but unused), missing (used/required but
+not specified), or incorrect (mismatching version between what's specified in
+`dist.ini` vs in source code, incorrect phase like test prereqs specified in
+runtime, etc).
 
-Designed to work with prerequisites that are manually written. Does not work if
-you use AutoPrereqs.
+Checking actual usage of prereqs is done using `Perl::PrereqScanner` (or
+`Perl::PrereqScanner::Lite`).
+
+Sections that will be checked for prereqs include `[Prereqs / *]`, as well as
+`OSPrereqs`, `Extras/lint-prereqs/Assume-*`. Designed to work with prerequisites
+that are manually written. Does not work if you use AutoPrereqs (using
+AutoPrereqs basically means that you do not specify prereqs and just use
+whatever modules are detected by the scanner.)
 
 Sometimes there are prerequisites that you know are used but can't be detected
-by scan_prereqs, or you want to include anyway. If this is the case, you can
-instruct lint_prereqs to assume the prerequisite is used.
+by the scanner, or you want to include anyway. If this is the case, you can
+instruct lint_prereqs to assume that the prerequisite is used.
 
-    ;!lint_prereqs assume-used # even though we know it is not currently used
+    ;!lint_prereqs assume-used "even though we know it is not currently used"
     Foo::Bar=0
-    ;!lint_prereqs assume-used # we are forcing a certain version
+
+    ;!lint_prereqs assume-used "we are forcing a certain version"
     Baz=0.12
 
 Sometimes there are also prerequisites that are detected by scan_prereqs, but
-you know are already provided by some other modules. So to make lint-prereqs
-ignore them:
+are false positives (`Perl::PrereqScanner::Lite` sometimes does this because its
+parser is simpler) or you know are already provided by some other modules. So to
+make lint-prereqs ignore them:
 
     [Extras / lint-prereqs / assume-provided]
     Qux::Quux=0
 
-By default, core modules are not allowed to be specified. Sometimes though you
-need to specify because on some systems some core modules are not available
-(e.g. they are split into separate OS packages). In this case, you can specify:
-
- ;!lint_prereqs allow-core
- Data::Dumper=0
-
 _
     args => {
         perl_version => {
@@ -138,7 +163,7 @@ _
         },
         lite => {
             schema => ['bool*'],
-            default => 1,
+            default => 0,
             summary => 'Use Perl::PrereqScanner::Lite instead of Perl::PrereqScanner',
             "summary.alt.bool.not" =>
                 'Use Perl::PrereqScanner instead of Perl::PrereqScanner::Lite',
@@ -187,37 +212,48 @@ sub lint_prereqs {
     my %mods_from_ini;
     my %assume_used;
     my %assume_provided;
-    my %allow_core;
-    for my $section (grep {
-        m!^(
-              osprereqs \s*/\s* .+ |
-              osprereqs(::\w+)+ |
-              prereqs (?: \s*/\s* \w+)? |
-              extras \s*/\s* lint[_-]prereqs \s*/\s* (allow-core|assume-(?:provided|used))
-          )$!ix}
-                         $cfg->list_sections) {
+    for my $section ($cfg->list_sections) {
+        $section =~ m!^(
+                          osprereqs \s*/\s* .+ |
+                          osprereqs(::\w+)+ |
+                          prereqs (?: \s*/\s* (?<prereqs_phase_rel>\w+))? |
+                          extras \s*/\s* lint[_-]prereqs \s*/\s* (assume-(?:provided|used))
+                      )$!ix or next;
+        #$log->errorf("TMP: section=%s, %%+=%s", $section, {%+});
+
+        my $phase;
+        if (my $pr = $+{prereqs_phase_rel}) {
+            if ($pr =~ /^(build|configure|develop|runtime|test)/i) {
+                $phase = ucfirst(lc($1));
+            } else {
+                return [400, "Invalid section '$section' (unknown phase)"];
+            }
+        } else {
+            $phase = "Runtime";
+        }
+
         for my $param ($cfg->list_keys($section)) {
             my $v         = $cfg->get_value($section, $param);
             my $dir = $cfg->get_directive_before_key($section, $param);
             my $dir_s = $dir ? join(" ", @$dir) : "";
             #$log->tracef("section=%s, v=%s, param=%s, directive=%s", $section, $param, $v, $dir_s);
-            if ($section =~ /allow-core/ || $dir_s =~ /^lint[_-]prereqs\s+allow-core\b/m) {
-                $allow_core{$param} = $v;
-            } else {
-                $mods_from_ini{$param}   = $v unless $section =~ /assume-provided/;
-                $assume_provided{$param} = $v if     $section =~ /assume-provided/;
-                $assume_used{$param}     = $v if     $section =~ /assume-used/ ||
-                    $dir_s =~ /^lint[_-]prereqs\s+assume-used\b/m;
-            }
-        }
-    }
+
+            $mods_from_ini{$phase}{$param}   = $v unless $section =~ /assume-provided/;
+            $assume_provided{$phase}{$param} = $v if     $section =~ /assume-provided/;
+            $assume_used{$phase}{$param}     = $v if     $section =~ /assume-used/ ||
+                $dir_s =~ /^lint[_-]prereqs\s+assume-used\b/m;
+        } # for param
+    } # for section
+    _create_prereqs_for_Any_phase(\%mods_from_ini);
+    _create_prereqs_for_Any_phase(\%assume_provided);
+    _create_prereqs_for_Any_phase(\%assume_used);
     $log->tracef("mods_from_ini: %s", \%mods_from_ini);
     $log->tracef("assume_used: %s", \%assume_used);
     $log->tracef("assume_provided: %s", \%assume_provided);
-    $log->tracef("allow_core: %s", \%allow_core);
 
-    # assume package names from filenames, should be better and scan using PPI
-    my %pkgs;
+    # get packages from current dist. assume package names from filenames,
+    # should be better and scan using PPI
+    my %dist_pkgs;
     find({
         #no_chdir => 1,
         wanted => sub {
@@ -228,18 +264,18 @@ sub lint_prereqs {
             $pkg =~ s!/!::!g;
             $pkg .= (length($pkg) ? "::" : "") . $_;
             $pkg =~ s/\.pm$//;
-            $pkgs{$pkg}++;
+            $dist_pkgs{$pkg}++;
         },
     }, "lib");
-    $log->tracef("Packages: %s", \%pkgs);
+    $log->tracef("Dist packages: %s", \%dist_pkgs);
 
     my %mods_from_scanned = _scan_prereqs(lite=>$args{lite});
     $log->tracef("mods_from_scanned: %s", \%mods_from_scanned);
 
-    if ($mods_from_ini{perl} && $mods_from_scanned{perl}) {
-        if (version_ne($mods_from_ini{perl}, $mods_from_scanned{perl})) {
-            return [500, "Perl version from dist.ini ($mods_from_ini{perl}) ".
-                        "and scan_prereqs ($mods_from_scanned{perl}) mismatch"];
+    if ($mods_from_ini{Any}{perl} && $mods_from_scanned{Any}{perl}) {
+        if (version_ne($mods_from_ini{Any}{perl}, $mods_from_scanned{Any}{perl})) {
+            return [500, "Perl version from dist.ini ($mods_from_ini{Any}{perl}) ".
+                        "and scan_prereqs ($mods_from_scanned{Any}{perl}) mismatch"];
         }
     }
 
@@ -248,14 +284,14 @@ sub lint_prereqs {
         $log->tracef("Will assume perl %s (via perl_version argument)",
                      $args{perl_version});
         $perlv = $args{perl_version};
-    } elsif ($mods_from_ini{perl}) {
+    } elsif ($mods_from_ini{Any}{perl}) {
         $log->tracef("Will assume perl %s (via dist.ini)",
-                     $mods_from_ini{perl});
-        $perlv = $mods_from_ini{perl};
-    } elsif ($mods_from_scanned{perl}) {
+                     $mods_from_ini{Any}{perl});
+        $perlv = $mods_from_ini{Any}{perl};
+    } elsif ($mods_from_scanned{Any}{perl}) {
         $log->tracef("Will assume perl %s (via scan_prereqs)",
-                     $mods_from_scanned{perl});
-        $perlv = $mods_from_scanned{perl};
+                     $mods_from_scanned{Any}{perl});
+        $perlv = $mods_from_scanned{Any}{perl};
     } else {
         $log->tracef("Will assume perl %s (from running interpreter's \$^V)",
                      $^V);
@@ -269,47 +305,76 @@ sub lint_prereqs {
     }
 
     my @errs;
-    for my $mod (keys %mods_from_ini) {
-        my $v = $mods_from_ini{$mod};
-        next if $mod eq 'perl';
-        $log->tracef("Checking mod from dist.ini: %s (%s)", $mod, $v);
-        my $is_core = Module::CoreList::More->is_still_core($mod, $v, $perlv);
-        if (!$args{core_prereqs} && $is_core && !exists($allow_core{$mod})) {
-            push @errs, {
-                module  => $mod,
-                req_v   => $v,
-                is_core => 1,
-                error   => "Core in perl ($perlv to latest) but ".
-                    "mentioned in dist.ini",
-                remedy  => "Remove from dist.ini",
-            };
-        }
-        my $scanv = $mods_from_scanned{$mod};
-        if (defined($scanv) && $scanv != 0 && version_ne($v, $scanv)) {
-            push @errs, {
-                module  => $mod,
-                req_v   => $v,
-                is_core => $is_core,
-                error   => "Version mismatch between dist.ini ($v) ".
-                    "and from scanned_prereqs ($scanv)",
-                remedy  => "Fix either the code or version in dist.ini",
-            };
-        }
-        unless (defined($scanv) || exists($assume_used{$mod})) {
-            push @errs, {
-                module  => $mod,
-                req_v   => $v,
-                is_core => $is_core,
-                error   => "Unused but listed in dist.ini",
-                remedy  => "Remove from dist.ini",
-            };
-        }
-    }
 
+    # check modules that are specified in dist.ini but extraneous (unused) or
+    # have mismatched version or phase
+    {
+        for my $mod (keys %{$mods_from_ini{Any}}) {
+            my $v = $mods_from_ini{Any}{$mod};
+            next if $mod eq 'perl';
+            $log->tracef("Checking mod from dist.ini: %s (%s)", $mod, $v);
+            my $is_core = Module::CoreList::More->is_still_core($mod, $v, $perlv);
+            if (!$args{core_prereqs} && $is_core) {
+                push @errs, {
+                    module  => $mod,
+                    req_v   => $v,
+                    is_core => 1,
+                    error   => "Core in perl ($perlv to latest) but ".
+                        "mentioned in dist.ini",
+                    remedy  => "Remove from dist.ini",
+                };
+            }
+            my $scanv = $mods_from_scanned{Any}{$mod};
+            if (defined($scanv) && $scanv != 0 && version_ne($v, $scanv)) {
+                push @errs, {
+                    module  => $mod,
+                    req_v   => $v,
+                    is_core => $is_core,
+                    error   => "Version mismatch between dist.ini ($v) ".
+                        "and from scanned_prereqs ($scanv)",
+                    remedy  => "Fix either the code or version in dist.ini",
+                };
+            }
+            if (defined($mods_from_scanned{Test}{$mod}) &&
+                    !defined($mods_from_scanned{Runtime}{$mod}) &&
+                    !defined($mods_from_ini{Test}{$mod}) &&
+                    defined($mods_from_ini{Runtime}{$mod})) {
+                push @errs, {
+                    module  => $mod,
+                    req_v   => $v,
+                    is_core => $is_core,
+                    error   => "Only used in test phase but listed under runtime prereq in dist.ini",
+                    remedy  => "Move prereq from runtime to test prereq in dist.ini",
+                };
+            }
+            if (defined($mods_from_scanned{Runtime}{$mod}) &&
+                    defined($mods_from_ini{Test}{$mod}) &&
+                    !defined($mods_from_ini{Runtime}{$mod})) {
+                push @errs, {
+                    module  => $mod,
+                    req_v   => $v,
+                    is_core => $is_core,
+                    error   => "Used in runtime phase but listed only under test prereq in dist.ini",
+                    remedy  => "Move prereq from test to runtime prereq in dist.ini",
+                };
+            }
+            unless (defined($scanv) || exists($assume_used{Any}{$mod})) {
+                push @errs, {
+                    module  => $mod,
+                    req_v   => $v,
+                    is_core => $is_core,
+                    error   => "Unused but listed in dist.ini",
+                    remedy  => "Remove from dist.ini",
+                };
+            }
+        }
+    } # END check modules from dist.ini
+
+    # check lumped modules
     {
         no strict 'refs';
         my %lumped_mods;
-        for my $mod (keys %mods_from_ini) {
+        for my $mod (keys %{$mods_from_ini{Any}}) {
             next unless $mod =~ /::Lumped$/;
             my $mod_pm = $mod;
             $mod_pm =~ s!::!/!g;
@@ -321,9 +386,9 @@ sub lint_prereqs {
         last unless %lumped_mods;
         $log->tracef("Checking lumped modules");
         for my $mod (keys %lumped_mods) {
-            my $v = $mods_from_ini{$mod};
+            my $v = $mods_from_ini{Any}{$mod};
             my $is_core = Module::CoreList::More->is_still_core($mod, $v, $perlv);
-            if (exists $mods_from_ini{$mod}) {
+            if (exists $mods_from_ini{Any}{$mod}) {
                 push @errs, {
                     module  => $mod,
                     req_v   => $v,
@@ -333,26 +398,37 @@ sub lint_prereqs {
                 };
             }
         }
-    }
+    } # END check lumped modules
 
-    for my $mod (keys %mods_from_scanned) {
-        next if $mod eq 'perl';
-        my $v = $mods_from_scanned{$mod};
-        $log->tracef("Checking mod from scanned: %s (%s)", $mod, $v);
-        my $is_core = Module::CoreList::More->is_still_core($mod, $v, $perlv);
-        next if exists $pkgs{$mod};
-        unless (exists($mods_from_ini{$mod}) ||
-                    exists($assume_provided{$mod}) ||
+    # check modules from scanned: check for missing prereqs (scanned/used but
+    # not listed in dist.ini).
+    {
+        for my $mod (keys %{$mods_from_scanned{Any}}) {
+            next if $mod eq 'perl';
+            my $v = $mods_from_scanned{Any}{$mod};
+            $log->tracef("Checking mod from scanned: %s (%s)", $mod, $v);
+            my $is_core = Module::CoreList::More->is_still_core($mod, $v, $perlv);
+            next if exists $dist_pkgs{$mod}; # skip modules from same dist
+            unless (exists($mods_from_ini{Any}{$mod}) ||
+                        exists($assume_provided{Any}{$mod}) ||
                         ($args{core_prereqs} ? 0 : $is_core)) {
-            push @errs, {
-                module  => $mod,
-                req_v   => $v,
-                is_core => $is_core,
-                error   => "Used but not listed in dist.ini",
-                remedy  => "Put '$mod=$v' in dist.ini",
-            };
+                my $phase;
+                for (qw/Runtime Test Build Configure Develop/) {
+                    if (defined $mods_from_scanned{$_}{$mod}) {
+                        $phase = $_;
+                        last;
+                    }
+                }
+                push @errs, {
+                    module  => $mod,
+                    req_v   => $v,
+                    is_core => $is_core,
+                    error   => "Used but not listed in dist.ini",
+                    remedy  => "Put '$mod=$v' in dist.ini (e.g. in [Prereqs/${phase}Requires])",
+                };
+            }
         }
-    }
+    } # END check modules from scanned
 
     @errs = sort {prereq_ala_perlancar($a->{module}, $b->{module})} @errs;
 
@@ -364,7 +440,7 @@ sub lint_prereqs {
 }
 
 1;
-# ABSTRACT: Check extraneous/missing prerequisites in dist.ini
+# ABSTRACT: Check extraneous/missing/incorrect prerequisites in dist.ini
 
 __END__
 
@@ -374,11 +450,11 @@ __END__
 
 =head1 NAME
 
-App::LintPrereqs - Check extraneous/missing prerequisites in dist.ini
+App::LintPrereqs - Check extraneous/missing/incorrect prerequisites in dist.ini
 
 =head1 VERSION
 
-This document describes version 0.36 of App::LintPrereqs (from Perl distribution App-LintPrereqs), released on 2016-02-27.
+This document describes version 0.37 of App::LintPrereqs (from Perl distribution App-LintPrereqs), released on 2016-03-01.
 
 =head1 SYNOPSIS
 
@@ -389,41 +465,41 @@ This document describes version 0.36 of App::LintPrereqs (from Perl distribution
 
 =head2 lint_prereqs(%args) -> [status, msg, result, meta]
 
-Check extraneous/missing prerequisites in dist.ini.
+Check extraneous/missing/incorrect prerequisites in dist.ini.
 
-Check C<[Prereqs / *]> (as well as C<OSPrereqs>, C<Extras/lint-prereqs/Assume-*>)
-sections in your C<dist.ini> against what's actually being used in your Perl code
-(using C<Perl::PrereqScanner::Lite>) and what's in Perl core list of modules.
-Will complain if your prerequisites are not actually used (extraneous
-dependencies), or if there are missing prerequisites. Will also complain if a
-prerequisite is already in Perl core (but this can be configured).
+lint-prereqs can improve your prereqs specification in C<dist.ini> by reporting
+prereqs that are extraneous (specified but unused), missing (used/required but
+not specified), or incorrect (mismatching version between what's specified in
+C<dist.ini> vs in source code, incorrect phase like test prereqs specified in
+runtime, etc).
 
-Designed to work with prerequisites that are manually written. Does not work if
-you use AutoPrereqs.
+Checking actual usage of prereqs is done using C<Perl::PrereqScanner> (or
+C<Perl::PrereqScanner::Lite>).
+
+Sections that will be checked for prereqs include C<[Prereqs / *]>, as well as
+C<OSPrereqs>, C<Extras/lint-prereqs/Assume-*>. Designed to work with prerequisites
+that are manually written. Does not work if you use AutoPrereqs (using
+AutoPrereqs basically means that you do not specify prereqs and just use
+whatever modules are detected by the scanner.)
 
 Sometimes there are prerequisites that you know are used but can't be detected
-by scan_prereqs, or you want to include anyway. If this is the case, you can
-instruct lint_prereqs to assume the prerequisite is used.
+by the scanner, or you want to include anyway. If this is the case, you can
+instruct lint_prereqs to assume that the prerequisite is used.
 
- ;!lint_prereqs assume-used # even though we know it is not currently used
+ ;!lint_prereqs assume-used "even though we know it is not currently used"
  Foo::Bar=0
- ;!lint_prereqs assume-used # we are forcing a certain version
+ 
+ ;!lint_prereqs assume-used "we are forcing a certain version"
  Baz=0.12
 
 Sometimes there are also prerequisites that are detected by scan_prereqs, but
-you know are already provided by some other modules. So to make lint-prereqs
-ignore them:
+are false positives (C<Perl::PrereqScanner::Lite> sometimes does this because its
+parser is simpler) or you know are already provided by some other modules. So to
+make lint-prereqs ignore them:
 
  [Extras / lint-prereqs / assume-provided]
  Qux::Quux=0
 
-By default, core modules are not allowed to be specified. Sometimes though you
-need to specify because on some systems some core modules are not available
-(e.g. they are split into separate OS packages). In this case, you can specify:
-
- ;!lint_prereqs allow-core
- Data::Dumper=0
-
 This function is not exported by default, but exportable.
 
 Arguments ('*' denotes required arguments):
@@ -438,7 +514,7 @@ If set to 0 (the default), will complain if there are prerequisites to core
 modules. If set to 1, prerequisites to core modules are required just like other
 modules.
 
-=item * B<lite> => I<bool> (default: 1)
+=item * B<lite> => I<bool> (default: 0)
 
 Use Perl::PrereqScanner::Lite instead of Perl::PrereqScanner.
 
